<?php
if ( ! class_exists('MobilePress_admin'))
{
	/**
	 * MobilePress class for creating the admin area
	 *
	 * @package MobilePress
	 * @since 1.0
	 */
	class MobilePress_admin {
		
		var $title;
		var $description;
		var $forcemobile;
		var $iswebbrowser_iphone;
		var $iswebbrowser_windows;
		var $iswebbrowser_operamini;
		var $currenttheme;
		var $section;
		
		/**
		 * Admin class constructor, sets up variables that we need
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function MobilePress_admin()
		{
			global $wpdb;
			
			$this->title					= mopr_get_option('title', 1);
			$this->description				= mopr_get_option('description', 1);
			$this->forcemobile				= mopr_convert_to_bool(mopr_get_option('forcemobile', 1));
			$this->iswebbrowser_iphone		= mopr_convert_to_bool(mopr_get_option('iswebbrowser_iphone', 1));
			$this->iswebbrowser_operamini	= mopr_convert_to_bool(mopr_get_option('iswebbrowser_operamini', 1));
			$this->iswebbrowser_windows		= mopr_convert_to_bool(mopr_get_option('iswebbrowser_windows', 1));
			
			if (isset($_GET['section']) && $_GET['page'] == 'mobilepress-themes')
			{
				$this->section		= $_GET['section'];
				$this->currenttheme	= mopr_get_option($_GET['section'], 2);
			}
			else
			{
				$this->section		= "defaulttheme";
				$this->currenttheme	= mopr_get_option($this->section, 2);
			}
		}
		
		/**
		 * Renders the MobilePress options page
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function render_options()
		{
			if (isset($_POST['save']))
			{
				global $wpdb;
				
				// Post data we need to save
				$updates	= array(
								'title'						=> $_POST['title'],
								'description'				=> $_POST['description'],
								'forcemobile'				=> $_POST['forcemobile'],
								'iswebbrowser_iphone'		=> $_POST['iswebbrowser_iphone'],
								'iswebbrowser_windows'		=> $_POST['iswebbrowser_windows'],
								'iswebbrowser_operamini'	=> $_POST['iswebbrowser_operamini']
							);
				
				// Update the sved data
				foreach ($updates as $name => $update)
				{
					$wpdb->query(
						$wpdb->prepare("
							UPDATE
								" . MOPR_TABLE . "
							SET
								option_value = '%s'
							WHERE
								option_name = '" . $name . "'
							", $update)
					);
				}
				
				// Reload changed variables after save
				$this->MobilePress_admin();
				
				$message = "<p>Options Saved!</p>";
				mopr_display_notice($message);
				
				$update_success = TRUE;
			}
			else if (isset($_POST['upgrade']))
			{
				require_once(MOPR_PATH . 'classes/install.php');
				$upgrade = new MobilePress_install;
				$upgrade->upgrade();
				
				$message = "<p>MobilePress Upgraded!</p>";
				mopr_display_notice($message);
				
				$update_success = TRUE;
			}
			else if (isset($_POST['create']))
			{
				require_once(MOPR_PATH . 'classes/install.php');
				$install = new MobilePress_install;
				
				// Reload changed variables after table install
				$this->MobilePress_admin();
				
				$message = "<p>MobilePress table has been created!</p>";
				mopr_display_notice($message);
				
				$update_success = TRUE;
			}
			else if (isset($_POST['add']))
			{
				require_once(MOPR_PATH . 'classes/install.php');
				$install = new MobilePress_install;
				$install->add_defaults();
				
				// Reload changed variables after table update
				$this->MobilePress_admin();
				
				$message = "<p>MobilePress options have been added to your table!</p>";
				mopr_display_notice($message);
				
				$update_success = TRUE;
			}
			
			$title = $this->title;
			$description = $this->description;
			
			// This isn't lekker - need to find a nicer way
			$forcemobile_selectedyes = $this->set_selected($this->forcemobile, 'yes');
			$forcemobile_selectedno = $this->set_selected($this->forcemobile, 'no');
			$iswebbrowser_iphone_selectedyes = $this->set_selected($this->iswebbrowser_iphone, 'yes');
			$iswebbrowser_iphone_selectedno = $this->set_selected($this->iswebbrowser_iphone, 'no');
			$iswebbrowser_windows_selectedyes = $this->set_selected($this->iswebbrowser_windows, 'yes');
			$iswebbrowser_windows_selectedno = $this->set_selected($this->iswebbrowser_windows, 'no');
			$iswebbrowser_operamini_selectedyes = $this->set_selected($this->iswebbrowser_operamini, 'yes');
			$iswebbrowser_operamini_selectedno = $this->set_selected($this->iswebbrowser_operamini, 'no');
			
			// Check if table exisits and if version is out dated, display appropriate message
			// Since we cannot redefine a constant (i.e. our version), we only run the checks if we have not just upgraded/updated
			if ( ! isset($update_success))
			{
				$this->initial_checks();
			}
			
			$content	= '<div class="wrap">';
			$content	.= '
							<form method="post" action="admin.php?page=mobilepress">
							<div id="icon-options-general" class="icon32"><br /></div>
							<h2>Options</h2>
							<table class="form-table">
								<tr>
									<th scope="row">Blog Title:</th>
									<td>
										<input type="text" name="title" value="' . $title . '" class="regular-text" /> <span class="description">Leave blank to default to your WordPress blog title</span>
									</td>
								</tr>
								<tr>
									<th scope="row">Blog Description:</th>
									<td>
										<input type="text" name="description" value="' . $description . '" class="regular-text" /> <span class="description">Leave blank to default to your WordPress blog description</span>
									</td>
								</tr>
								<tr>
									<th scope="row">Force Mobile Site?</th>
									<td>
										<select name="forcemobile">
											<option value="false"' . $forcemobile_selectedno . '>No</option>
											<option value="true"' . $forcemobile_selectedyes . '>Yes</option>
										</select>
									</td>
								</tr>
								<tr>
									<th scope="row">Treat iPhone as a web browser?</th>
									<td>
										<select name="iswebbrowser_iphone">
											<option value="false"' . $iswebbrowser_iphone_selectedno . '>No</option>
											<option value="true"' . $iswebbrowser_iphone_selectedyes . '>Yes</option>
										</select>
									</td>
								</tr>
								<tr>
									<th scope="row">Treat Windows smartphone as a web browser?</th>
									<td>
										<select name="iswebbrowser_windows">
											<option value="false"' . $iswebbrowser_windows_selectedno . '>No</option>
											<option value="true"' . $iswebbrowser_windows_selectedyes . '>Yes</option>
										</select>
									</td>
								</tr>
								<tr>
									<th scope="row">Treat Opera Mini as a web browser?</th>
									<td>
										<select name="iswebbrowser_operamini">
											<option value="false"' . $iswebbrowser_operamini_selectedno . '>No</option>
											<option value="true"' . $iswebbrowser_operamini_selectedyes . '>Yes</option>
										</select>
									</td>
								</tr>
							</table>
							<p class="submit"><input type="submit" name="save" class="button-primary" value="Save Changes!" /></p>
							</form>
						';
			$content	.= "</div>";
			
			echo $content;
		}
		
		/**
		 * Renders the themes page
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function render_themes()
		{
			// Find the page title
			if ($this->section == "defaulttheme")
			{
				$pagetitle = "Default Browser";
			}
			else if ($this->section == "iphonetheme")
			{
				$pagetitle = "iPhone Browser";
			}
			else if ($this->section == "operamtheme")
			{
				$pagetitle = "Opera Mini Browser";
			}
			else if ($this->section == "windowstheme")
			{
				$pagetitle = "Windows Mobile Browser";
			}
			
			// Do we need to activate a theme?
			if (isset($_GET['action']) == "activate")
			{
				global $wpdb;
					
				$template	= $_GET['template'];
				$theme		= $_GET['theme'];
				
				$wpdb->query(
					$wpdb->prepare("
						UPDATE
							" . MOPR_TABLE . "
						SET
							option_value = '%s',
							option_value2 = '%s'
						WHERE
							option_name = '%s'
						", $template, $theme, $this->section)
				);
				
				// Reload variables after activation
				$this->MobilePress_admin();
				
				$message = "<p>" . $theme . " has been activated for " . $pagetitle . "!</p>";
				
				mopr_display_notice($message);
			}
			
			// Check if table exisits and if version is out dated, display appropriate message
			$this->initial_checks();
		?>
		
		<div class="wrap">
			<div id="icon-themes" class="icon32"><br /></div>
			<h2>Mobile Themes</h2>
			<form action="admin.php" method="get">
			<table class="form-table">
				<tr>
					<td>
						<input type="hidden" name="page" value="mobilepress-themes">
						<select name="section" id="theme">
							<option value="defaulttheme"<?php if($this->section == "defaulttheme") { echo " selected"; } ?>>Default Browser</option>
							<option value="iphonetheme"<?php if($this->section == "iphonetheme") { echo " selected"; } ?>>iPhone Browser</option>
							<option value="operamtheme"<?php if($this->section == "operamtheme") { echo " selected"; } ?>>Opera Mini Browser</option>
							<option value="windowstheme"<?php if($this->section == "windowstheme") { echo " selected"; } ?>>Windows Mobile Browser</option>
						</select>
						<input type="submit" name="select" value="Select!" class="button" />
						(Select the browser you want to choose a theme for)
					</td>
				</tr>
			</table>
			<p class="submit"></p>
			</form>
		
			<?php
				// Make sure themes exist
				if($this->check_for_themes())
				{
			?>
				<h2>Current Theme Set For "<?php echo $pagetitle; ?>"</h2>
				<?php $this->current_theme(); ?>
				
				<h2>Select A New Theme For "<?php echo $pagetitle; ?>"</h2>
				<?php $this->available_themes(); ?>
			<?php
				}
			?>
		
		</div>
			
		<?php
		}
		
		/**
		 * Checks whether we need to do any updates, an installation etc
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function initial_checks()
		{
			// If the table exists
			if (mopr_check_table_exists())
			{
				// Check to see if the options table is empty
				if (mopr_check_empty_table())
				{
					// If the table empty, throw message to add options
					$message = '
								<form method="post" action="admin.php?page=mobilepress">
									<p>MobilePress table is empty! <input type="submit" name="add" value="Add Options!" class="button" /></p>
								</form>
							';
							
					mopr_display_notice($message);
				}
				else
				{
					// Check if an upgrade is needed
					if (MOPR_DBVERSION < MOPR_VERSION)
					{
						$message = '
									<form method="post" action="admin.php?page=mobilepress">
										<p>Your MobilePress install is not the latest version! <input type="submit" name="upgrade" value="Upgrade!" class="button" /></p>
									</form>
								';
								
						mopr_display_notice($message);
					}
				}
			}
			else
			{
				// If the table doesnt exist throw message to create it
				$message = '
							<form method="post" action="admin.php?page=mobilepress">
								<p>MobilePress table does not exist! <input type="submit" name="create" value="Create!" class="button" /></p>
							</form>
						';
						
				mopr_display_notice($message);
			}
		}
	
		/**
		 * Checks to see if an option must be selected
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING Whether selected or not 
		 */
		function set_selected($force, $check)
		{
			if ($force == true && $check == "yes")
			{
				return " selected";
			}
			else if ($force == false && $check == "no")
			{
				return " selected";
			}
			
			return;
		}
	
		/**
		 * Core WP function for getting themes the themes directory (with a few modifications)
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function get_themes() {
		
			global $wp_themes, $wp_broken_themes;

			if ( isset($wp_themes) )
				return $wp_themes;

			$themes = array();
			$wp_broken_themes = array();
			$theme_loc = $theme_root = MOPR_ROOT_PATH . "themes";
			if ( '/' != WP_CONTENT_DIR ) // don't want to replace all forward slashes, see Trac #4541
				$theme_loc = str_replace(WP_CONTENT_DIR, '', $theme_root);

			// Files in wp-content/themes directory and one subdir down
			$themes_dir = @ opendir($theme_root);
			if ( !$themes_dir )
				return false;

			while ( ($theme_dir = readdir($themes_dir)) !== false ) {
				if ( is_dir($theme_root . '/' . $theme_dir) && is_readable($theme_root . '/' . $theme_dir) ) {
					if ( $theme_dir{0} == '.' || $theme_dir == '..' || $theme_dir == 'CVS' )
						continue;
					$stylish_dir = @ opendir($theme_root . '/' . $theme_dir);
					$found_stylesheet = false;
					while ( ($theme_file = readdir($stylish_dir)) !== false ) {
						if ( $theme_file == 'style.css' ) {
							$theme_files[] = $theme_dir . '/' . $theme_file;
							$found_stylesheet = true;
							break;
						}
					}
					@closedir($stylish_dir);
					if ( !$found_stylesheet ) { // look for themes in that dir
						$subdir = "$theme_root/$theme_dir";
						$subdir_name = $theme_dir;
						$theme_subdir = @ opendir( $subdir );
						while ( ($theme_dir = readdir($theme_subdir)) !== false ) {
							if ( is_dir( $subdir . '/' . $theme_dir) && is_readable($subdir . '/' . $theme_dir) ) {
								if ( $theme_dir{0} == '.' || $theme_dir == '..' || $theme_dir == 'CVS' )
									continue;
								$stylish_dir = @ opendir($subdir . '/' . $theme_dir);
								$found_stylesheet = false;
								while ( ($theme_file = readdir($stylish_dir)) !== false ) {
									if ( $theme_file == 'style.css' ) {
										$theme_files[] = $subdir_name . '/' . $theme_dir . '/' . $theme_file;
										$found_stylesheet = true;
										break;
									}
								}
								@closedir($stylish_dir);
							}
						}
						@closedir($theme_subdir);
						$wp_broken_themes[$theme_dir] = array('Name' => $theme_dir, 'Title' => $theme_dir, 'Description' => __('Stylesheet is missing.'));
					}
				}
			}
			if ( is_dir( $theme_dir ) )
				@closedir( $theme_dir );

			if ( !$themes_dir || !$theme_files ) 
				return $themes;

			sort($theme_files);

			foreach ( (array) $theme_files as $theme_file ) {
				if ( !is_readable("$theme_root/$theme_file") ) {
					$wp_broken_themes[$theme_file] = array('Name' => $theme_file, 'Title' => $theme_file, 'Description' => __('File not readable.'));
					continue;
				}

				$theme_data = get_theme_data("$theme_root/$theme_file");

				$name        = $theme_data['Name'];
				$title       = $theme_data['Title'];
				$description = wptexturize($theme_data['Description']);
				$version     = $theme_data['Version'];
				$author      = $theme_data['Author'];
				$template    = $theme_data['Template'];
				$stylesheet  = dirname($theme_file);

				$screenshot = false;
				foreach ( array('png', 'gif', 'jpg', 'jpeg') as $ext ) {
					if (file_exists("$theme_root/$stylesheet/screenshot.$ext")) {
						$screenshot = "screenshot.$ext";
						break;
					}
				}

				if ( empty($name) ) {
					$name = dirname($theme_file);
					$title = $name;
				}

				if ( empty($template) ) {
					if ( file_exists(dirname("$theme_root/$theme_file/index.php")) )
						$template = dirname($theme_file);
					else
						continue;
				}

				$template = trim($template);

				if ( !file_exists("$theme_root/$template/index.php") ) {
					$parent_dir = dirname(dirname($theme_file));
					if ( file_exists("$theme_root/$parent_dir/$template/index.php") ) {
						$template = "$parent_dir/$template";
					} else {
						$wp_broken_themes[$name] = array('Name' => $name, 'Title' => $title, 'Description' => __('Template is missing.'));
						continue;
					}
				}

				$stylesheet_files = array();
				$stylesheet_dir = @ dir("$theme_root/$stylesheet");
				if ( $stylesheet_dir ) {
					while ( ($file = $stylesheet_dir->read()) !== false ) {
						if ( !preg_match('|^\.+$|', $file) && preg_match('|\.css$|', $file) )
							$stylesheet_files[] = "$theme_loc/$stylesheet/$file";
					}
				}

				$template_files = array();
				$template_dir = @ dir("$theme_root/$template");
				if ( $template_dir ) {
					while(($file = $template_dir->read()) !== false) {
						if ( !preg_match('|^\.+$|', $file) && preg_match('|\.php$|', $file) )
							$template_files[] = "$theme_loc/$template/$file";
					}
				}

				$template_dir = dirname($template_files[0]);
				$stylesheet_dir = dirname($stylesheet_files[0]);

				if ( empty($template_dir) )
					$template_dir = '/';
				if ( empty($stylesheet_dir) )
					$stylesheet_dir = '/';

				// Check for theme name collision.  This occurs if a theme is copied to
				// a new theme directory and the theme header is not updated.  Whichever
				// theme is first keeps the name.  Subsequent themes get a suffix applied.
				// The Default always trump their pretenders.
				if ( isset($themes[$name]) ) {
					if ( ('Default' == $name) &&
							 ('default' == $stylesheet) ) {
						// If another theme has claimed to be one of our default themes, move
						// them aside.
						$suffix = $themes[$name]['Stylesheet'];
						$new_name = "$name/$suffix";
						$themes[$new_name] = $themes[$name];
						$themes[$new_name]['Name'] = $new_name;
					} else {
						$name = "$name/$stylesheet";
					}
				}

				$themes[$name] = array('Name' => $name, 'Title' => $title, 'Description' => $description, 'Author' => $author, 'Version' => $version, 'Template' => $template, 'Stylesheet' => $stylesheet, 'Template Files' => $template_files, 'Stylesheet Files' => $stylesheet_files, 'Template Dir' => $template_dir, 'Stylesheet Dir' => $stylesheet_dir, 'Status' => $theme_data['Status'], 'Screenshot' => $screenshot, 'Tags' => $theme_data['Tags']);
			}

			// Resolve theme dependencies.
			$theme_names = array_keys($themes);

			foreach ( (array) $theme_names as $theme_name ) {
				$themes[$theme_name]['Parent Theme'] = '';
				if ( $themes[$theme_name]['Stylesheet'] != $themes[$theme_name]['Template'] ) {
					foreach ( (array) $theme_names as $parent_theme_name ) {
						if ( ($themes[$parent_theme_name]['Stylesheet'] == $themes[$parent_theme_name]['Template']) && ($themes[$parent_theme_name]['Template'] == $themes[$theme_name]['Template']) ) {
							$themes[$theme_name]['Parent Theme'] = $themes[$parent_theme_name]['Name'];
							break;
						}
					}
				}
			}

			$wp_themes = $themes;

			return $themes;
		}
	
		/**
		 * WordPress core function to check if themes exist and if default theme exists. @TODO: May need reworking in a future release
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function check_for_themes() {
			$themes = $this->get_themes();
		
			if ( empty($themes) ) {
				$message = "<p>Please upload a theme to your mobilepress themes directory!</p>";
				mopr_display_notice($message);
				return false;
			} else {
				$default = $themes['Default']['Title'];
			
				if ( empty($default) ) {
					$message = "<p>You need to upload the default theme!</p>";
					mopr_display_notice($message);
					return false;
				} else {
					return true;
				}
			}
		}
	
		/**
		 * Core WP function to get the info from the current active theme (made a few modifications)
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function current_theme_info() {
			$themes = $this->get_themes();
			$current_theme = $this->currenttheme;
			$ct->name = $current_theme;
			$ct->title = $themes[$current_theme]['Title'];
			$ct->version = $themes[$current_theme]['Version'];
			$ct->parent_theme = $themes[$current_theme]['Parent Theme'];
			$ct->template_dir = $themes[$current_theme]['Template Dir'];
			$ct->stylesheet_dir = $themes[$current_theme]['Stylesheet Dir'];
			$ct->template = $themes[$current_theme]['Template'];
			$ct->stylesheet = $themes[$current_theme]['Stylesheet'];
			$ct->screenshot = $themes[$current_theme]['Screenshot'];
			$ct->description = $themes[$current_theme]['Description'];
			$ct->author = $themes[$current_theme]['Author'];
			$ct->tags = $themes[$current_theme]['Tags'];
			return $ct;
		}
	
		/**
		 * WP Core code for displaying current theme (with a few modifications)
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function current_theme() {
			$ct = $this->current_theme_info();
		?>
			<div id="current-theme">
			<?php if ( $ct->screenshot ) : ?>
			<img src="<?php echo WP_CONTENT_URL . $ct->stylesheet_dir . '/' . $ct->screenshot; ?>" alt="Current theme preview" />
			<?php endif; ?>
			<h3><?php printf(_c('%1$s %2$s by %3$s|1: theme title, 2: theme version, 3: theme author'), $ct->title, $ct->version, $ct->author) ; ?></h3>
			<p class="description"><?php echo $ct->description; ?></p>
			<?php if ($ct->parent_theme) { ?>
				<p><?php printf(__('The template files are located in <code>%2$s</code>.  The stylesheet files are located in <code>%3$s</code>.  <strong>%4$s</strong> uses templates from <strong>%5$s</strong>.  Changes made to the templates will affect both themes.'), $ct->title, $ct->template_dir, $ct->stylesheet_dir, $ct->title, $ct->parent_theme); ?></p>
			<?php } else { ?>
				<p><?php printf(__('All of this theme&#8217;s files are located in <code>%2$s</code>.'), $ct->title, $ct->template_dir, $ct->stylesheet_dir); ?></p>
			<?php } ?>
			<?php if ( $ct->tags ) : ?>
			<p><?php _e('Tags:'); ?> <?php echo join(', ', $ct->tags); ?></p>
			<?php endif; ?>
			</div>
		<?php
		}
	
		/**
		 * WP Core code for displaying list of available themes (with a few modifications)
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function available_themes() {
			$themes = get_themes();
			ksort( $themes );
		?>
		<br class="clear" />
		<table id="availablethemes" cellspacing="0" cellpadding="0">
		<?php
		$style = '';

		$theme_names = array_keys($themes);
		natcasesort($theme_names);

		$rows = ceil(count($theme_names) / 3);
		for ( $row = 1; $row <= $rows; $row++ )
			for ( $col = 1; $col <= 3; $col++ )
				$table[$row][$col] = array_shift($theme_names);

		foreach ( $table as $row => $cols ) {
		?>
		<tr>
		<?php
		foreach ( $cols as $col => $theme_name ) {
			$class = array('available-theme');
			if ( $row == 1 ) $class[] = 'top';
			if ( $col == 1 ) $class[] = 'left';
			if ( $row == $rows ) $class[] = 'bottom';
			if ( $col == 3 ) $class[] = 'right';
		?>
			<td class="<?php echo join(' ', $class); ?>">
		<?php if ( !empty($theme_name) ) :
			$template = $themes[$theme_name]['Template'];
			$stylesheet = $themes[$theme_name]['Stylesheet'];
			$title = $themes[$theme_name]['Title'];
			$version = $themes[$theme_name]['Version'];
			$description = $themes[$theme_name]['Description'];
			$author = $themes[$theme_name]['Author'];
			$screenshot = $themes[$theme_name]['Screenshot'];
			$stylesheet_dir = $themes[$theme_name]['Stylesheet Dir'];
			$tags = $themes[$theme_name]['Tags'];
			$activate_link = "admin.php?page=mobilepress-themes&amp;section=".urlencode($this->section)."&amp;action=activate&amp;template=".urlencode($template)."&amp;theme=".urlencode($title);
			$activate_text = attribute_escape( sprintf( __('Activate "%s"'), $title ) );
		?>
				<a href="<?php echo $activate_link; ?>" class="theme screenshot">
		<?php if ( $screenshot ) : ?>
					<img src="<?php echo WP_CONTENT_URL . $stylesheet_dir . '/' . $screenshot; ?>" alt="" />
		<?php endif; ?>
				</a>
				<h3><a href="<?php echo $activate_link; ?>"><?php echo $title; ?></a></h3>
				<p><?php echo $description; ?></p>
		<?php if ( $tags ) : ?>
				<p><?php _e('Tags:'); ?> <?php echo join(', ', $tags); ?></p>
		<?php endif; ?>
		<?php endif; // end if not empty theme_name ?>
			</td>
		<?php } // end foreach $cols ?>
		</tr>
		<?php } // end foreach $table ?>
		</table>
	
		<?php
		}
	}
}
?>
