<?php
if ( ! class_exists('MobilePress_check'))
{
	/**
	 * Class that does all the checks to determine if we are dealing with a Mobile browser
	 *
	 * @package MobilePress
	 * @since 1.0
	 */
	class MobilePress_check {
		
		/**
		 * @var string $browser Type of Mobile Browser
		 */
		var $browser = "mobile";
		
		/**
		 * @var bool $activated If the mobile theme is active
		 */
		var $activated = FALSE;
		
		/**
		 * @var string $theme The mobile theme to render
		 */
		var $theme;
		
		/**
		 * Initialize the checking of the mobile browse
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function init()
		{
			// Get the theme we need to render
			$this->theme = mopr_get_option('defaulttheme',1);
			
			// If viewing the mobile website
			switch(TRUE)
			{	
				// ?mobile accesses the mobile version of the website
				case (isset($_GET['mobile'])):
					$this->browser = "mobile";
					$this->activated = TRUE;
					break;
					
				// If forcing iphone theme
				case (isset($_GET['iphone'])):
					$this->browser = "iphone";
					$this->activated = TRUE; 
					$this->theme = mopr_get_option('iphonetheme',1);
					break;
					
				// If forcing opera mini theme
				case (isset($_GET['operam'])):
					$this->browser = "operam";
					$this->activated = TRUE; 
					$this->theme = mopr_get_option('operamtheme',1);
					break;
					
				// If forcing windows mobile theme
				case (isset($_GET['msce'])):
					$this->browser = "msce";
					$this->activated = TRUE; 
					$this->theme = mopr_get_option('windowstheme',1);
					break;
				
				// ?nomobile renders the orignial website
				case (isset($_GET['nomobile'])):
					$this->activated = FALSE;
					$this->theme = '';
					break;
					
				// Apple browser renders as mobile
				case (eregi('apple', $_SERVER['HTTP_USER_AGENT']) && eregi('mobile',$_SERVER['HTTP_USER_AGENT']) && ! mopr_convert_to_bool(mopr_get_option('iswebbrowser_iphone',1))):
					$this->browser = "iphone";
					$this->activated = TRUE;
					$this->theme = mopr_get_option('iphonetheme',1);
					break;
				
				// Render mobile site if viewing from and iphone
				case (eregi('iPhone', $_SERVER['HTTP_USER_AGENT']) && ! mopr_convert_to_bool(mopr_get_option('iswebbrowser_iphone', 1))):
					$this->browser = "iphone";
					$this->activated = TRUE; 
					$this->theme = mopr_get_option('iphonetheme',1);
					break;
				
				// Render mobile site if Opera mini
				case (eregi('opera mini', $_SERVER['HTTP_USER_AGENT']) && ! mopr_convert_to_bool(mopr_get_option('iswebbrowser_operamini', 1))):
					$this->browser = "operam";
					$this->activated = TRUE;
					$this->theme = mopr_get_option('operamtheme',1);
					break;
				
				// Render mobile site if Windows Mobile
				case (eregi('Windows CE', $_SERVER['HTTP_USER_AGENT']) && ! mopr_convert_to_bool(mopr_get_option('iswebbrowser_windows',1))):
					$this->browser = "msce";
					$this->activated = TRUE;
					$this->theme = mopr_get_option('windowstheme',1);
					break;
					
				// Other mobile browsers render as mobile
				case (preg_match('/(up.browser|up.link|mmp|symbian|smartphone|midp|wap|phone|vodafone|o2|pocket|mobile|pda|psp|treo|htc|palm|palmos|blackberry)/i',strtolower($_SERVER['HTTP_USER_AGENT']))): // check for tell-tale signs
					$this->browser = "mobile";
					$this->activated = TRUE;
					break;
					
				// Wap browser
				case (((strpos(strtolower($_SERVER['HTTP_ACCEPT']),'text/vnd.wap.wml') > 0) || (strpos(strtolower($_SERVER['HTTP_ACCEPT']),'application/vnd.wap.xhtml+xml')>0)) || ((isset($_SERVER['HTTP_X_WAP_PROFILE']) || isset($_SERVER['HTTP_PROFILE'])))):
					$this->activated = TRUE;
					break;
				
				// Shortend user agents
				case (in_array(strtolower(substr($_SERVER['HTTP_USER_AGENT'],0,4)),array('acs-'=>'acs-','alav'=>'alav','alca'=>'alca','amoi'=>'amoi','audi'=>'audi','aste'=>'aste','avan'=>'avan','benq'=>'benq','bird'=>'bird','blac'=>'blac','blaz'=>'blaz','brew'=>'brew','cell'=>'cell','cldc'=>'cldc','cmd-'=>'cmd-','dang'=>'dang','doco'=>'doco','eric'=>'eric','hipt'=>'hipt','inno'=>'inno','ipaq'=>'ipaq','java'=>'java','jigs'=>'jigs','kddi'=>'kddi','keji'=>'keji','leno'=>'leno','lg-c'=>'lg-c','lg-d'=>'lg-d','lg-g'=>'lg-g','lge-'=>'lge-','maui'=>'maui','maxo'=>'maxo','midp'=>'midp','mits'=>'mits','mmef'=>'mmef','mobi'=>'mobi','mot-'=>'mot-','moto'=>'moto','mwbp'=>'mwbp','nec-'=>'nec-','newt'=>'newt','noki'=>'noki','opwv'=>'opwv','palm'=>'palm','pana'=>'pana','pant'=>'pant','pdxg'=>'pdxg','phil'=>'phil','play'=>'play','pluc'=>'pluc','port'=>'port','prox'=>'prox','qtek'=>'qtek','qwap'=>'qwap','sage'=>'sage','sams'=>'sams','sany'=>'sany','sch-'=>'sch-','sec-'=>'sec-','send'=>'send','seri'=>'seri','sgh-'=>'sgh-','shar'=>'shar','sie-'=>'sie-','siem'=>'siem','smal'=>'smal','smar'=>'smar','sony'=>'sony','sph-'=>'sph-','symb'=>'symb','t-mo'=>'t-mo','tim-'=>'tim-','tosh'=>'tosh','treo'=>'treo','tsm-'=>'tsm-','upg1'=>'upg1','upsi'=>'upsi','vk-v'=>'vk-v','voda'=>'voda','wap-'=>'wap-','wapa'=>'wapa','wapi'=>'wapi','wapp'=>'wapp','wapr'=>'wapr','webc'=>'webc','winw'=>'winw','winw'=>'winw','xda-'=>'xda-'))):
					$this->browser = "mobile";
					$this->activated = TRUE;
					break;
					
				// Render mobile site for mobile search engines
				case (eregi('Googlebot-Mobile', $_SERVER['HTTP_USER_AGENT']) || eregi('YahooSeeker/M1A1-R2D2', $_SERVER['HTTP_USER_AGENT'])):
					$this->browser = "mobile";
					$this->activated = TRUE;
					break;
			}
			
			$this->set_sessions();
		}
		
		/**
		 * Sets the sessions needed for rendering a mobile website
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function set_sessions()
		{
			$_SESSION['SESS_MOBILE_BROWSER'] 	= $this->browser;
			$_SESSION['SESS_MOBILE_ACTIVE'] 	= $this->activated;
			$_SESSION['SESS_MOBILE_THEME'] 		= $this->theme;
		}
		
	}
}
?>
