<?php
if ( ! class_exists('MobilePress_core'))
{
	/**
	 * The core MobilePress class where the magic happens
	 *
	 * @package MobilePress
	 * @since 1.0
	 */
	class MobilePress_core {
		
		/**
		 * Loads the install class and setups the plugin including database creation
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function load_activation()
		{
			require_once(MOPR_PATH . 'classes/install.php');
			$install = new MobilePress_install;
		}
		
		/**
		 * Deactivates the plugin
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function load_deactivation()
		{
			// Shutdown the plugin (nothing here yet)
		}
		
		/**
		 * Calls the method to create admin menus and to set up the admin area
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function create_admin()
		{
			// Action to initiate the admin setup and admin menu setup
			add_action('admin_menu', array(&$this, 'load_admin'));
		}
		
		/**
		 * Creates the MobilePress menus and creates an admin object which creates the menu content
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function load_admin()
		{
			require_once(MOPR_PATH . 'classes/admin.php');
			
			$MobilePress_admin = new MobilePress_admin;
			
			// Add admin 'MobilePress' to the top level menu area
			add_menu_page('MobilePress', 'MobilePress', 10, 'mobilepress', '', WP_PLUGIN_URL . '/mobilepress/system/images/icon.png');
			
				//Add the 'Options' submenu to the MobilePress menu and render the page
				add_submenu_page('mobilepress', 'MobilePress Options', 'Options', 10, 'mobilepress', array(&$MobilePress_admin, 'render_options'));
				
				//Add the 'Themes' submenu to the MobilePress menu and render the page
				add_submenu_page('mobilepress', 'MobilePress Themes', 'Themes', 10, 'mobilepress-themes', array(&$MobilePress_admin, 'render_themes'));
		}
		
		/**
		 * Does the checks and decides whether to render a mobile or normal website
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function load_site()
		{
			if (isset($_GET['killsession']))
			{
				session_unset();
				session_destroy();
			}
			
			// Plugin preference is set to render entire site in mobile.
			if (mopr_convert_to_bool(mopr_get_option('forcemobile', 1)))
			{
				$_SESSION['SESS_MOBILE_ACTIVE'] = TRUE;
			}
			
			// Check if mobile sesison var exists
			// Also, check if ?mobile or ?nomobile is set. If so, establish the session var so that 
			// subsequent page calls will render in the desired mode.
			// This means ?mobile only needs to be called once, and existing links don't need to be altered.
			if (
				( ! isset($_SESSION['SESS_MOBILE_ACTIVE']) || (trim($_SESSION['SESS_MOBILE_ACTIVE']) == ''))
				|| (isset($_GET['mobile']))
				|| (isset($_GET['nomobile']))
			)
			{
				// If the session var doesn't exist, proceed to init and check to see if we are dealing with a mobile browser
				require_once(MOPR_PATH . 'classes/check.php');
				$checkmobile = new MobilePress_check;
				$checkmobile->init();
				// SESS_MOBILE_ACTIVE var is now created - either TRUE or FALSE
				// Additionally, if TRUE, SESS_MOBILE_THEME will be created with a string value
			}
			
			// Set the browser variable
			$browser = $_SESSION['SESS_MOBILE_BROWSER'];
			
			// This is for testing mobile themes in a normal browser and mobile browsers
			// Resets browser session if you are switching between mobile browsers
			if (
				(isset($_GET['mobile']) && ($browser != "mobile"))
				|| (isset($_GET['iphone']) && ($browser != "iphone"))
				|| (isset($_GET['operam']) && ($browser != "operam"))
				|| (isset($_GET['msce']) && ($browser != "msce"))
			)
			{
				require_once(MOPR_PATH . 'classes/check.php');
				$checkmobile = new MobilePress_check;
				$checkmobile->init();
				
				// Reset the browser variable
				$browser = $_SESSION['SESS_MOBILE_BROWSER'];
			}
			
			if ($_SESSION['SESS_MOBILE_ACTIVE'] === TRUE)
			{
				// Double check session var for theme
				// Fall back on default if any problems
				if ( ! isset($_SESSION['SESS_MOBILE_THEME']) || (trim($_SESSION['SESS_MOBILE_THEME']) == ''))
				{
					$_SESSION['SESS_MOBILE_THEME'] = mopr_get_option('defaulttheme', 1);
				}

				// SESS_MOBILE_THEME && SESS_MOBILE_ACTIVE are both set - Render time!
				require_once(MOPR_PATH . 'classes/render.php');
				$render = new MobilePress_render($browser);
				$render->init();
			}
			else
			{
				// SESS_MOBILE_ACTIVE has explicitly been set to false. Either by following nomobile link, or there were no matches in the detection script (We're using a web browser)
				// Nothing to do here at the momnet.
			}
		}

	}
}
?>
