<?php
if ( ! class_exists('MobilePress_install'))
{
	/**
	 * Class that deals with installing the MobilePress plugin
	 *
	 * @package MobilePress
	 * @since 1.0
	 */
	class MobilePress_install {
		
		/**
		 * Start the installation process
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function MobilePress_install()
		{	
			// If MobilePress table exists then try upgrade, otherwise create it and add defaults (cant use CheckTable() in functions.php in this code)
			if (mopr_check_table_exists())
			{
				$this->upgrade();
			}
			else
			{
				$this->setup_table();
				$this->add_defaults();
			}
		}
		
		/**
		 * Creates the MobilePress Table
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function setup_table()
		{
			$sql	= "
						CREATE TABLE " . MOPR_TABLE . " (
							id mediumint(9) NOT NULL AUTO_INCREMENT,
							option_name VARCHAR(100) NOT NULL,
							option_value VARCHAR(100) NOT NULL,
							option_value2 VARCHAR(100) NOT NULL,
						UNIQUE KEY id (id))
						ENGINE = MYISAM 
						CHARACTER SET utf8 
						COLLATE utf8_unicode_ci;
					";
					
			// Require upgrade.php from the CORE
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
		
		/**
		 * Upgrades the plugin database if it is outdated
		 *
		 * @package MobilePress
		 * @since 1.0.2
		 */
		function upgrade()
		{	
			global $wpdb;
			
			if (MOPR_DBVERSION <= '1.0.2')
				$this->upgrade_102();
			
			// Change the version in the database to the latest version after upgrade
			$wpdb->query(
				$wpdb->prepare("
					UPDATE
						" . MOPR_TABLE . "
					SET
						option_value = '%s'
					WHERE
						option_name = 'version'
					", MOPR_VERSION)
				);
		}
		
		/**
		 * Inserts default values into the MobilePress database
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function add_defaults()
		{
			global $wpdb;
			
			// Folder names
			$defaulttemplate 	= $wpdb->escape('default');
			$windowstemplate 	= $wpdb->escape('default');
			$iphonetemplate 	= $wpdb->escape('iphone');
			$operamtemplate 	= $wpdb->escape('default');
	
			// Descriptive theme names
			$defaulttheme 		= $wpdb->escape('Default');
			$windowstheme 		= $wpdb->escape('Default');
			$iphonetheme 		= $wpdb->escape('iPhone');
			$operamtheme 		= $wpdb->escape('Default');
			
			$sql = "INSERT INTO " . MOPR_TABLE . " (
						option_name, 
						option_value,
						option_value2
					) 
					VALUES 
						('version', '" . MOPR_VERSION . "', ''), 
						('defaulttheme', '$defaulttemplate', '$defaulttheme'), 
						('windowstheme', '$windowstemplate', '$windowstheme'), 
						('iphonetheme', '$iphonetemplate', '$iphonetheme'), 
						('operamtheme', '$operamtemplate', '$operamtheme'), 
						('title', '', ''), 
						('description', '', ''), 
						('iswebbrowser_iphone', 'false', ''), 
						('iswebbrowser_windows', 'false', ''), 
						('iswebbrowser_operamini', 'false', ''), 
						('forcemobile', 'false', '')
					";
	
			$results = $wpdb->query($sql);
		}
		
		/**
		 * Critical upgrades for plugin version 1.0.2 and lower
		 *
		 * @package MobilePress
		 * @since 1.0.2
		 */
		function upgrade_102()
		{
			global $wpdb;
			
			// First delete any duplicate DB entries that may have been caused by a bug prior to version 1.1
			$sql	= "DELETE FROM " . MOPR_TABLE . " WHERE id > 11";
			$delete	= $wpdb->query($sql);
			
			// Reset auto increment
			$sql	= "ALTER TABLE " . MOPR_TABLE . " AUTO_INCREMENT = 1";
			$reset	= $wpdb->query($sql);
		}
		
	}
}
?>
