<?php
if ( ! class_exists('MobilePress_render'))
{
	/**
	 * Class that deals with all aspects of rendering the mobile website
	 *
	 * @package MobilePress
	 * @since 1.0
	 */
	class MobilePress_render {
	
		/**
		 * @var string $defaulttheme The default mobile theme
		 */
		var $defaulttheme;
		
		/**
		 * @var string $title Custom title for the mobile blog
		 */
		var $title;
		
		/**
		 * @var string $description Custom description for the mobile blog
		 */
		var $description;
		
		/**
		 * Constructor which sets up the variables we will need to use
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @param STRING $browser The browser to render for
		 */
		function MobilePress_render($browser)
		{
			$this->defaulttheme	= mopr_get_option('defaulttheme',1);
			$this->title		= mopr_get_option('title', 1);
			$this->description	= mopr_get_option('description', 1);
			$this->usingcache	= mopr_convert_to_bool(mopr_get_option('usingcache', 1));
			$this->browser		= $browser;
		}
		
		/**
		 * Initialize the rendering of the mobile website
		 *
		 * @package MobilePress
		 * @since 1.0
		 */
		function init()
		{	
			add_filter('stylesheet', array(&$this, 'set_stylesheet'));
			add_filter('theme_root', array(&$this, 'set_theme_root'));
			add_filter('theme_root_uri', array(&$this, 'set_theme_uri'));
			add_filter('template', array(&$this, 'set_template'));
			
			if ($this->title != '')
			{
				add_filter('option_blogname', array(&$this, 'set_title'));
			}
			
			if ($this->description != '')
			{
				add_filter('option_blogdescription', array(&$this, 'set_description'));
			}
		}
		
		/**
		 * Sets the stylesheet to the themes mobile stylesheet
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING Name of the theme where the stylesheet will be called
		 */
		function set_stylesheet()
		{	
			if ( ! isset($_SESSION['SESS_MOBILE_THEME']) || trim($_SESSION['SESS_MOBILE_THEME']) == '')
			{
				$stylesheet = $this->defaulttheme;
			}
			else
			{
				$stylesheet = $_SESSION['SESS_MOBILE_THEME'];
			}
			
			return $stylesheet;
		}
		
		/**
		 * Sets the theme root to the MobilePress theme directory
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING Root directory of the MobilePress theme directory
		 */
		function set_theme_root()
		{
			$theme_root = MOPR_ROOT_PATH . "themes";
			return $theme_root;	
		}
		
		/**
		 * Sets the path to the themes directory
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING The MobilePress theme directory
		 */
		function set_theme_uri()
		{
			$theme_uri = get_bloginfo('wpurl') . "/wp-content/plugins/mobilepress/themes";
			return $theme_uri;
		}
		
		/**
		 * Sets the blogs template to the MobilePress template
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING The name of the MobilePress template to be used for rendering
		 */
		function set_template()
		{
			if ( ! isset($_SESSION['SESS_MOBILE_THEME']) || trim($_SESSION['SESS_MOBILE_THEME']) == '')
			{
				$template = $this->defaulttheme;
			}
			else
			{
				$template = $_SESSION['SESS_MOBILE_THEME'];
			}
			
			return $template;
		}
		
		/**
		 * Returns the mobile blogs title
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING The mobile blogs title
		 */
		function set_title()
		{
			return $this->title;
		}
		
		/**
		 * Returns the mobile blogs description
		 *
		 * @package MobilePress
		 * @since 1.0
		 * @return STRING The mobile blogs description
		 */
		function set_description()
		{
			return $this->description;
		}
		
	}
}
?>
